import sys
from os import remove, write
from threading import Thread

#from gevent import sleep
from time import sleep
import requests
import bs4

from .config import IMAGE_CACHE, SINGLE_IMAGE_DELETE_AFTER_SECS

def delete_file(path):
    sleep(SINGLE_IMAGE_DELETE_AFTER_SECS)
    print('Erasing', path)
    try:
        remove(path)
    except FileNotFoundError:
        pass


def error(msg):
    sys.stderr.write(msg + "\n")
    sys.stderr.flush()

def get(url: str, write_dir: str, delete=True):
    orig_url = url
    if not url.startswith('https://imgur.com/'):
        url = 'https://imgur.com' + url
    found_url = ''
    found_urls = []
    found_list_file = ''

    album = False
    if "gallery" in url:
        url = url.replace("gallery", "a")
    if "/a/" in url:
        album = True
        if not url.endswith("blog"):
            url += "/layout/blog"


    if not album:
        print('Getting img', url)
        url = 'https://i.imgur.com/' + url.rsplit('/', 1)[-1]
        with open(f'{write_dir}/{url[-11:]}', 'wb') as img:
            img.write(requests.get(url).content)
        if delete:
            Thread(target=delete_file, args=[f"{write_dir}/{url[-11:]}"]).start()
    else:
        print('Detecting album/gallery images (contentUrl)', url)
        soup = bs4.BeautifulSoup(requests.get(url).text, 'html.parser')
        for count, el in enumerate(soup.select('.post-image meta[itemprop="contentUrl"]'), start=1):
            try:
                found_url = "https:" + el['content']
                if '?1' in found_url:
                    continue
            except KeyError:
                error("Could not obtain url for detected image (contentUrl)")
                continue
            if found_url.endswith('ico.jpg'):
                continue
            found_urls.append(found_url[-11:])
            write(count, found_url, write_dir, delete)
        if len(found_urls) == 0:
            print('Detecting album/gallery images (id)', url)
            for count, el in enumerate(soup.select('.post-image-container'), start=1):
                try:
                    found_url = "https://i.imgur.com/" + el['id'] + ".jpg" # equivalent to .png
                except KeyError:
                    error("Could not obtain url for detected image (id)")
                    continue
                found_urls.append(found_url[-11:])
                write(count, found_url, write_dir, delete)
        # Write the found urls to a file with the name of the album so the viewer endpoint can get them
        found_list_file = write_dir + orig_url.replace('/', '_')
        with open(found_list_file, 'w') as f:
            f.write(','.join(found_urls))
        Thread(target=delete_file, args=[found_list_file]).start()

def write(count: int, found_url: str, write_dir: str, delete=True):
    print(f"Downloading image {count}: {found_url}")

    print("Writing image", f"{write_dir}{found_url[-11:]}")
    with open(f"{write_dir}{found_url[-11:]}", "wb") as f:
        f.write(requests.get(found_url).content)

    if delete:
        Thread(target=delete_file, args=[f"{write_dir}{found_url[-11:]}"]).start()
